#' convert camera intrinsic parameters to/from matrix/list
#' @family lensDistortion-internal
#' @param ... the input to convert. A named list or vector or just all the parameters.
#'             In the latter case, order is cx,cy,fx,fy.
#' @param return.what a string, being 'list' or 'matrix'.
#'                    if 'matrix', returns   fx  0 cx
#'                                            0 fy cy
#'                                            0  0  1
#'                    if 'list', returns list(cx=,cy=,fx=,fy=).
#' @examples \dontrun{
#' convertDistortionCoefficients( cx,cy,fx,fy, return.what='matrix' )
#' convertDistortionCoefficients( c(cx,cy,fx,fy), return.what='list' )
#' convertDistortionCoefficients( list(cx=10,cy=10,fx=10,fy=10), return.what='matrix' )
#' }
#' @family lensDistortion-internal
#' @export
#' @keywords internal
convertIntrinsicParameters <- function( ..., return.what='matrix' )
{
    obj <- list(...)
    if ( length(obj)==1 ) obj <- obj[[1]]

    if ( is.matrix(obj) && return.what == 'matrix' )
        return(obj)
    if ( is.matrix(obj) && return.what != 'matrix' )
        return( list( fx=obj[1,1],
                      fy=obj[2,2],
                      cx=obj[1,3],
                      cy=obj[2,3]
                      ))
    if ( is.list(obj) && return.what == 'matrix' ) {
        if ( is.null(names(obj)) ) names(obj) <- c("cx","cy","fx","fy")
        return( matrix( c( obj$fx, 0, obj$cx, 
                           0, obj$fy, obj$cy,
                           0,      0,      1 ), byrow=T, nrow=3, ncol=3 ) )
    }
    if ( is.list(obj) && return.what != 'matrix' ) {
        if ( is.null(names(obj)) ) names(obj) <- c("cx","cy","fx","fy")
        return(obj)
    }
}

#' convert distortion coefficients to/from vector/list
#' @param ... the input to convert. A named list or vector or just all the parameters.
#' @param return.what a string, being 'list' or 'vector'.
#'                    if 'vector', returns c(k1,k2,k3,p1,p2).
#'                    if 'list', returns list(k1=,k2=,k3=,p1=,p2=).
#' @examples \dontrun{
#' convertDistortionCoefficients( k1,k2,k3,p1,p2, return.what='list' )
#' convertDistortionCoefficients( c(k1,k2,k3,p1,p2), return.what='list' )
#' convertDistortionCoefficients( list(k1=0,k2=0,k3=0,p1=0,p2=0), return.what='vector' )
#' }
#' @family lensDistortion-internal
#' @export
#' @keywords internal
convertDistortionCoefficients <- function( ..., return.what='list' )
{
    obj <- list(...)
    if ( length(obj)==1 ) obj <- obj[[1]]

    if ( is.vector(obj) && return.what=='vector' )
        return(obj)
    if ( is.vector(obj) && return.what!='vector' )
        return( list( k1=obj[1],
                      k2=obj[2],
                      k3=obj[3],
                      p1=obj[4],
                      p2=obj[5] ) )

    if ( is.null(names(obj)) ) names(obj) <- c("k1","k2","k3","p1","p2")
    if ( is.list(obj) && return.what=='vector' )
        return( as.vector(unlist( obj[[c("k1","k2","k3","p1","p2")]] )) )
    else
        return(obj)
}
