#' generate a new optimal camera matrix
#' @param cameraMatrix current camera matrix
#' @param distCoefs (k1,k2,k3,p1,p2)
#' @param imgSize   (x,y)??
#' @param alpha if 0, new image is maximal internal image (ie excludes
#' borders). If 1, new image is the minimum bounding rectangle for the
#' entire distorted region (ie includes black borders).
#' @param newImgSize new image size (x,y)??
#' @return camera matrix: 3x3:
#' \deqn{
#'  \begin{array}{ccc}
#'  f_x &  0  & c_x\\
#'   0  & f_y & c_y\\
#'   0  &  0  &  1
#'  \end{array}}
#' @seealso Open CV page for this function...
#' @export
getOptimalNewCameraMatrix <- function( cameraMatrix, distCoefs, imgSize, alpha, newImgSize=imgSize )
{
    # get maximal inside rectangle and minimal bounding rectangle, in normalised coordinates (indep of cam matrix)
    # --- cvUndistortPoints( pts, &pts_out, cameramatrix, distCoeffs, 0, 0 ) --- #
    N <- 9
    pts <- t(expand.grid( (0:(N-1))/(N-1)*imgSize[1], (0:(N-1))/(N-1)*imgSize[2] ))

    o_pts <- radialDistortedToUndistortedOpenCV(pts, 
                                                distCoefs[1], distCoefs[2], distCoefs[3], distCoefs[4], distCoefs[5],
                                                cameraMatrix[1,3], cameraMatrix[2,3],
                                                cameraMatrix[1,1], cameraMatrix[2,2],
                                                one.based.input=F, one.based.output=F,
                                                scale.back=F,iters=5)
    if ( nrow(o_pts)>2 && ncol(o_pts)==2 ) o_pts <- t(o_pts)

#    iters <- 5
#    cxy <- c( cameraMatrix[1,3], cameraMatrix[2,3] )
#    fxy <- c( cameraMatrix[1,1], cameraMatrix[2,2] )
#
#    xys <- (pts-cxy)/fxy # 2 x n
#    xy0s <- xys
#
#    for ( j in 1:iters) {
#        r2s <- colSums(xys^2) # a VECTOR of r^2s
#        ks  <- ((distCoefs[3]*r2s+distCoefs[2])*r2s+distCoefs[1])*r2s+1 # VECTOR of distCoefs's
#        if ( distCoefs[4] == 0 && distCoefs[5] == 0 ) {
#            dxs <- 0
#        } else {
#            t2 <- xys[1,]*xys[2,]
#            dxs <- 2*distCoefs[4:5]*rbind(t2,t2) + distCoefs[5:4]*(rbind(r2s,r2s)+2*xys^2)
#        }
#        xys <- (xy0s-dxs)/rbind(ks,ks)  # NOTE: do I need to shift the *fx + cx ???
#    }
#    o_pts <- xys
    # -------------------------------------------------------------------------- #

    # ------- Find inside/outside rectangle 
    # icvGetRectangles( cameraMatrix, distCoeffs, 0, 0, imgSize, &inner, &outer );
    # find inside rectangle.
    ox0 <- min(o_pts[1,])
    oy0 <- min(o_pts[2,])
    ox1 <- max(o_pts[1,])
    oy1 <- max(o_pts[2,])
   
    ix0 <- max( o_pts[1, pts[1,]==0] )           # pts[1,]==0 is 1:N:end
    ix1 <- min( o_pts[1, pts[1,]==imgSize[1]] )  # N:N:end
    iy0 <- max( o_pts[2, pts[2,]==0] )           # 1:N
    iy1 <- min( o_pts[2, pts[2,]==imgSize[2]] )  # (end-N+1):end

    inner <- list( position=c(ix0,iy0), size=c(ix1-ix0,iy1-iy0) )
    outer <- list( position=c(ox0,oy0), size=c(ox1-ox0,oy1-oy0) ) # I think outer is wrong?

    # -------------------------------------------------------------------------- #


    # f,c if inner rectangle was the entire image [alpha=0]
    f0 <- (newImgSize-1)/inner$size
    c0 <- -f0*inner$position

    # f,c if outer rectangle was the entire image [alpha=1]
    f1 <- (newImgSize-1)/outer$size
    c1 <- (-f1*outer$position)

    # interpolate between the two
    newCameraMatrix <- matrix( 0, nrow=3, ncol=3 )
    fnew <- f0*(1-alpha) + f1*alpha
    cnew <- c0*(1-alpha) + c1*alpha
    newCameraMatrix[1,1] <- fnew[1]
    newCameraMatrix[2,2] <- fnew[2]
    newCameraMatrix[1,3] <- cnew[1]
    newCameraMatrix[2,3] <- cnew[2]
    newCameraMatrix[3,3] <- 1

    return( newCameraMatrix )
}
