# radialDistortedToUndistortedFulla.q v1.2
#' Convert radially distorted to undistorted coords w/ Hugin/Fulla.
##############################################################################
#' Given distorted image coords (0-based) and distortion parameters,
#'  returns the corresponding image coords in the UNdistorted image.
#' Parrots fulla, i.e. solves (1b) for r.
#'
#' The outputs are not necessarily integers.
#'
# Usage:
#       pts_u  <- radialDistortedToUndistortedFulla( pts_d, a,b,c,d, nx,ny, e,f, one.based.input=F, one.based.output=F )
#   where:
#' @inheritParams oneBasedSwitch
#@param i_pts    image coordinates for the distorted image (0-based).
#                 These should be an array/matrix of 2x[however many points],
#                 with x on the top and y on the bottom (or left/right).
#' @param a        parameter for distortion polynomial, r^3 term (see details)
#' @param b        parameter for distortion polynomial, barrel distort/r^2 term (see details)
#' @param c        parameter for distortion polynomial, r term (see details)
#' @param d        parameter for distortion polynomial, scaling term (see details)
#' @param nx       dimension of input image (x)
#'                   (only handles same size input/output image at the moment)
#' @param ny       dimension of input image (y)
#'                   (only handles same size input/output image at the moment)
#' @param e        the x shift of optical centre in the image 
#'                 Default image centre, leave if you don't know.
#' @param f        the y shift of optical centre in the image 
#'                 Default image centre, leave if you don't know.
#@param one.based.input  boolean, whether input coords are 1-based or 0-based. Default 0-based.
#@param one.based.output boolean, whether output coords are to be 1-based or 0-based. Default 0-based.
# Value:
#' @return pts_u   0-based pixel coords in the undistorted image corresponding to
#'                 the distorted ones provided. Matrix, 2x[npoints].
#'
#' @details
# Details (same as that for unBarrelDistortFulla):
#'
#'   This function solves following equation for r (same as hugin/panotools):
#'
#'       [xy]_{distorted} = [xy](d + cr + br^2 + ar^3),                 ---------- (1a)
#'                               equivalently
#'          r_{distorted} = r(d + cr + br^2 + ar^3),                    ---------- (1b)
#'
#'   where r is the location of a point in the UNdistorted/corrected image,
#'    being the normalised radius from the image centre.
#'   Also, d is often chosen to be 1-a-b-c, as this preserves image size: 
#'    when r = 1, then the distorted r is also 1.
#'
#'   That is, given r_{distorted}( xd, yd ), it finds r such that (1b) holds.
#'
#'   Once this r found, the scaling factor (d+cr+br^2+ar^3) in (1a) is calculated by
#'    rearranging (1b): it is equal to r_{distorted}/r.
#'
#'   Then (1a) is used to calculated [xy] by 
#'       
#'               [xy] = [xy]_{distorted} * r/r_{distorted}              ---------- (2)  
#'
#'   r is a normalised radius, such that the largest circle fitting entirely in the
#'    image has r=1.
#'  
#'   That is, given a (0-based) pixel index (xd,yd) in the distorted/uncorrected image,
#'    one calculates the radius rd by:
#'
#'      c_x       <- width/2                 # location of image centre before shifting
#'      c_y       <- height/2 
#'
#'      xd_shifted <- xd - c_x - e + 0.5     # add 0.5 because of centre/edge of pixel thing.
#'      yd_shifted <- yd - c_y - f + 0.5
#'
#'      R         <- MIN( width/2, height/2 )
#'      rd        <- sqrt( xd_shifted^2 + yd_shifted^2 ) / R
#'      r         <- { SOLVE (1b) for r }
#'  
#'      k         <- r/rd                    # [ this is just 1/(d + cr + br^2 + ar^3) ]
#'
#'      x         <- k*xd_shifted + c_x + e - 0.5      # scale by k and shift back.
#'      y         <- k*yd_shifted + c_y + f - 0.5      
#'   
#'   The return value of the function is (xd,yd), the distorted pix coords.
#'
#'   Then the intensity at location (x,y) in the undistorted image should be the intensity
#'    at location (xd,yd) in the distorted image.
#'   Interpolate it how you will.
#'
# See Also:
#' @references
#'   For the model, fulla source is incredibly helpful, and also the sites
#'       http://wiki.panotools.org/Lens_Correction_in_PanoTools
#'       http://wiki.panotools.org/Lens_correction_model
#'
#'   Someone has implemented these in Matlab (slightly different method???)
#'       http://staff.science.uva.nl/~iesteban/
#'
#'   For a way to determine a,b,c,d you need to have some sort of calibration image with known
#'    straight lines in it, then select points on that line, and basically optimise
#'    equation (1) with respect to a,b,c,d so that the undistorted line is straight.
#'
#'   Hugin is a useful tool for doing this, you essentially pick points on straight lines and optimise
#'    with respect to a,b,c only (some call for y,p,r first, then a,b,c ?)
#'
#'   NOTE: put d = 1 - a - b - c? this means that when r=1, r_dist = 1 -- preserves image size.
#'
#  Related Functions:
# @seealso
#    \code{\link{radialDistortedToUndistortedFulla} -- does the inverse of this function.
#    \code{\link{unBarrelDistortFulla}              -- removes barrel distortion in an entire image.
# 
#' @family radialDistortion
#' @export
#  AC 5 July 2011
##############################################################################
#' @section History:
#' v1.2 (7-11-2011)
#'   - added doxygen
#'   - added one based input/output options.
#' v1.1 (6-7-2011)
#'   - vectorising was broken, fixed it.
#' v1.0 (5-7-2011)
##############################################################################
radialDistortedToUndistortedFulla <- function( i_pts, a,b,c,d, nx,ny, e=0,f=0, one.based.input=F, one.based.output=F )
{
    # take a shortcut if there is no barrel distortion.
    if ( a==0 && b==0 && c==0 && d==1 ) 
        return( oneBasedSwitch(i_pts,one.based.input,one.based.output) )

    EPSTOL      <- 1e-14   # .Machine$double.eps
    pts <- i_pts
    D <- dim(i_pts)
    if ( D[1] != 2 && D[2] == 2 ) pts <- t(i_pts) # flip it if it's the wrong way round.
    # need 2x[npoints]

    cs <- c(e,f)     # centre shift
    TX <- c(nx,ny)/2
    R  <- min(TX)

    tmp <- pts - cs - TX + .5 - as.integer(one.based.input) # make 0-based
    rds <- sqrt(colSums(tmp^2))/R  # a VECTOR of rd's

    # construct idx_reconstruct such that if:
    # rds_unique <- unique(rds), then
    # rds_unique[idx_reconstruct] == rds
    idx_reconstruct <- pmatch(rds,(rds <- unique(rds)),duplicates.ok=TRUE)

    # solve poly rd = ar^4 + br^3 + cr^2 + dr  for r (r being coords in the undistorted image)
    # ru  <- polyroot(c(-rd,d,c,b,a))
    # return a [numroots]x[ncol(i_pts)] of solutions.
    #rus <- sapply( rds, function(rd) return(polyroot(c(-rd,d,c,b,a))) )
    coefs <- c(d,c,b,a)
    rus <- vapply( rds, function(rd) return(polyroot(c(-rd,coefs))),
                  complex(length.out=max(which(coefs!=0))) )

    # take the ones with no imaginary part that is closest to the starting value and positive.
    # ORIGINAL
    # apply( idx, 2, function(col) if ( sum(col)==1 ) return( which(col) )
    # else { which.min(abs( rus[,col]-ORIGINAL ))  } ?????
    idx <- Re(rus)>0 & abs(Im(rus)) <= EPSTOL
    sols <- colSums(idx)
    # note: idx0 ??
    idx1 <- which(sols==1)
    idx2 <- which(sols>1)
    # faster way to do this?
    sols[idx1] <- Re(rus[,idx1][idx[,idx1]])
    sols[idx2] <- vapply( idx2,
                   function(j) 
                     Re(rus[ which(idx[,j])[
                              which.min(abs(Re(rus[idx[,j],j])-rds[j]))
                             ],j
                           ]), -1 )
     rm(rus)

    # ru  <- Re(ru[abs(Im(ru))<=EPSTOL]) # Is Machine$double.eps too small...
    # ru  <- ru[ru>0] # positive roots
    # ru  <- ifelse( length(ru)==0, 0, ru[which.min(abs(ru-rd))] )
 
    ks  <- sols/rds

    # NOW NEED TO EXPAND BACK OUT !!!
    
    ptd <- t(t(tmp)*ks[idx_reconstruct]) + cs + TX - .5 + as.integer(one.based.output)
    # ptd <- k*tmp + cs + TX - .5 
    return(ptd)
}

