# radialDistortedToUndistortedOpenCV.q v1.0
#' Converts distorted coords to undistorted coords using OpenCV model.
##############################################################################
#' @details
#' Given undistorted image coords and distortion parameters,
#'  returns the corresponding image coords in the distorted image.
#' Parrots openCV model [eqn (1) in details].
#'
#' @param i_pts input points, distorted.
#' @inheritParams radialDistortedToUndistortedFulla
#' @inheritParams radialUndistortedToDistortedOpenCV
#' @param iters number of iterations to perform.
#' @param scale.back whether to scale the coords back to camera-specific cords
#'                   or leave in normalised. For internal use only.
#' @return pts_d undistorted pixel coords (not necessarily integers) corresponding to the input ones.
#'               It is up to the user to interpolate them. Matrix, 2x[npoints].
#
# Details:
#'@details
#'   This function reverses the model described in \code{\link{radialUndistortedToDistortedOpenCV}}.
#'   That is, given DISTORTED coordinates (u,v), the corresponding UNDISTORTED points (u',v') are givenby:
#'
#'      x'' <- (u - cx)/fx                               # normalise points
#'      y'' <- (v - cy)/fy
#'
#'      x'  <- undistort( x'', dist_coeffs )
#'      y'  <- undistort( y'', dist_coeffs )
#'
#'      u'  <- x'*fxscale + cxscale
#'      v'  <- y'*fxscale + cyscale
#'
#'   Here, the function undistort(x'',y'',dist_coeffs) should reverse the equations:
#'     
#'      x'' = kx' + dx,
#'      y'' = ky' + dy,
#'
#'      where:
#'      r2        <- x'^2 + y'^2
#'      k         <- (1 + k1(r2) + k2(r2)^2 + k3(r2)^3)  # scale factor
#'      dx        <- 2p1*x*y + p2(r2 + 2x^2)             # tangential in x
#'      dy        <- 2p2*x*y + p1(r2 + 2y^2)             # tangential in y
#'
#'   The intensity at (u',v') in the undistorted image corresponds to (u,v) in the distorted image.
#'   Interpolate it how you will.
#'
#'   Note that fxscale and cxscale have the effect of changing the field of view
#'    of the output image: for example when c.scale == c. and f.scale == f. then
#'    the output image contains no unmapped pixels and is the same size as the input.
#'   One can have c.scale and f.scale larger than c. and f. such that the output image
#'    shows the distorted edges of the input image and has unmapped pixels.
#' 
#'   To calculate these new values one uses the openCV function cv::getOptimalNewCameraMatrix().
#'
# See Also:
#'@references
#' OpenCV function cv2.undistortPoints
#'     http://opencv.itseez.com/modules/imgproc/doc/geometric_transformations.html?highlight=undistort#undistortpoints
#
#' @family radialDistortion
#' @export
# AC 9 Nov 2011
##############################################################################
#'@section History:
#' v1.0 (9-11-2011)
##############################################################################
# cat(' pts_d <- radialUndistortedToDistortedOpenCV <- function( i_pts, k1,k2,k3,p1,p2,fx,fy,cx,cy, one.based.input=FALSE, one.based.output=FALSE )\n')
radialDistortedToUndistortedOpenCV <- function( i_pts,k1,k2,k3,p1,p2,cx,cy,fx,fy,cxscale=cx,cyscale=cy,fxscale=fx,fyscale=fy, one.based.input=FALSE, one.based.output=FALSE,iters=5, scale.back=T )
{
    if ( k1 == 0 && k2 == 0 && k3 == 0 && p1 == 0 && p2 == 0 )
        return( oneBasedSwitch(i_pts, one.based.input, one.based.output) )
    pts <- i_pts - as.integer(one.based.input) # convert to zero-based.
    D <- dim(i_pts)
    if ( D[1] != 2 && D[2] == 2 ) pts <- t(i_pts) # flip it if it's the wrong way round.
   
    x0 <- x <- (pts - c(cx,cy))/c(fx,fy)

    # xd <- undistort( xdd, dist_coeffs )
    for ( j in 1:iters) {
        r2s <- colSums(x^2) 
        ks  <- ((k3*r2s+k2)*r2s+k1)*r2s+1 
        if ( p1 == 0 && p2 == 0 ) {
            dxs <- 0
        } else {
            t2 <- x[1,]*x[2,]
            #dxs <- 2*c(p1,p2)*rbind(t2,t2) + c(p2,p1)*(rbind(r2s,r2s)+2*x^2)
            dxs <- rbind(  2*p1*t2 + p2*r2s, 2*p2*t2 + p1*r2s ) +  2*c(p2,p1)*x^2
        }
        x <- (x0-dxs)/rbind(ks,ks)
    }

    if ( scale.back )
        pts_o <- x*c(fxscale,fyscale)+c(cxscale,cyscale) + as.integer(one.based.output)
    else
        pts_o <- x + as.integer(one.based.output)

    return(pts_o)
}
