# radialUndistortedToDistortedFulla.q v1.0
#' Convert undistorted to radially distorted Hugin/Fulla.
##############################################################################
#' Given undistorted image coords (0-based) and distortion parameters,
#'  returns the corresponding image coords in the distorted image.
#' Parrots fulla, i.e. implements eqn (1) in Details.
#'
#' pts is an array/matrix of 2x[however many points], x top, y bottom.
#' It takes 0 based image coords as inputs and returns the 0 based image
#'  coords that you should interpolate the intensity from.
#' The outputs are not necessarily integers.
#
# Usage:
#       pts_d  <- radialUndistortedToDistortedFulla( pts_u, a,b,c,d, nx,ny, e,f, one.based.input=F, one.based.output=F )
#   where:
#       i_pts   : image coordinates for the undistorted image (0-based).
#                 These should be an array/matrix of 2x[however many points],
#                 with x on the top and y on the bottom (or left/right).
#       a,b,c,d : parameters for distortion polynomial (see details)
#       nx,ny   : dimensions of the input/output image 
#                   (only handles same size input/output image at the moment)
#       e,f     : the shift of optical centre in the image 
#                 Default image centre, leave if you don't know.
#' @inheritParams radialDistortedToUndistortedFulla
# Value:
#' @return pts_u   0-based pixel coords in the undistorted image corresponding to
#'                 the distorted ones provided.
#
# Details (same as that for unBarrelDistortFulla):
#' @details
#'   This function implements the equation (same as hugin/panotools):
#'
#'       [xy]_{distorted} = [xy](d + cr + br^2 + ar^3),                 ---------- (1)
#'
#'   where r is the location of a point in the UNdistorted/corrected image,
#'    being the normalised radius from the image centre.
#'   Also, d is often chosen to be 1-a-b-c, as this preserves image size: 
#'    when r = 1, then the distorted r is also 1.
#'
#'   The normalised radius is such that the largest circle fitting entirely in the
#'    image has r=1.
#'  
#'   That is, given a (0-based) pixel index (x,y) in the distorted/uncorrected image,
#'    one calculates the radius r by:
#'
#'      c_x       <- width/2                 # location of image centre before shifting
#'      c_y       <- height/2 
#'
#'      x_shifted <- x - c_x - e + 0.5       # add 0.5 because of centre/edge of pixel thing.
#'      y_shifted <- y - c_y - f + 0.5
#'
#'      R         <- MIN( width/2, height/2 )
#'      r         <- sqrt( x_shifted^2 + y_shifted^2 ) / R
#'  
#'      k         <- (d + cr + br^2 + ar^3)  # scale factor
#'
#'      xd        <- k*x_shifted + c_x + e - 0.5      # scale by k and shift back.
#'      yd        <- k*y_shifted + c_y + f - 0.5      
#'   
#'   The return value of the function is (xd,yd), the distorted pix coords.
#'
#'   Then the intensity at location (x,y) in the undistorted image should be the intensity
#'    at location (xd,yd) in the distorted image.
#'   Interpolate it how you will.
#
# See Also:
#' @references
#'   For the model, fulla source is incredibly helpful, and also the sites
#'       http://wiki.panotools.org/Lens_Correction_in_PanoTools
#'       http://wiki.panotools.org/Lens_correction_model
#'
#'   Someone has implemented these in Matlab (slightly different method???)
#'       http://staff.science.uva.nl/~iesteban/
#'
#'   For a way to determine a,b,c,d you need to have some sort of calibration image with known
#'    straight lines in it, then select points on that line, and basically optimise
#'    equation (1) with respect to a,b,c,d so that the undistorted line is straight.
#'
#'   Hugin is a useful tool for doing this, you essentially pick points on straight lines and optimise
#'    with respect to a,b,c only (some call for y,p,r first, then a,b,c ?)
#'   I only optimised w.r.t. b so far (that is the parameter that corresponds directly to barrel distortion).
#'
#'   NOTE: put d = 1 - a - b - c? this means that when r=1, r_dist = 1 -- preserves image size.
#
# Related Functions:
#   radialDistortedToUndistortedFulla  -- does the inverse of this function.
#   unBarrelDistortFulla               -- removes barrel distortion in an entire image.
#' @family radialDistortion
#' @export
#
# AC 5 July 2011
##############################################################################
#' @section History:
#' v1.1 (7-11-2011)
#'    - added roxygen
#'    - added one based input/output options
#' v1.0 (5-7-2011)
##############################################################################
# cat('  pts_d  <- radialUndistortedToDistortedFulla( pts_u, a,b,c,d, nx,ny, e=0,f=0 )\n')
radialUndistortedToDistortedFulla <- function( i_pts, a,b,c,d, nx,ny, e=0,f=0, one.based.input=F, one.based.output=F )
{
    if ( a == 0 && b == 0 & c == 0 & d == 1 )
        return( oneBasedSwitch(i_pts,one.based.input,one.based.output) )
    pts <- i_pts - as.integer(one.based.input)
    D <- dim(i_pts)
    if ( D[1] != 2 && D[2] == 2 ) pts <- t(i_pts) # flip it if it's the wrong way round.
    
    cs <- c(e,f)     # centre shift
    TX <- c(nx,ny)/2
    R  <- min(TX)

    tmp <- pts - cs - TX + .5
    rs  <- sqrt(colSums(tmp^2))/R # a VECTOR of r's
    ks  <- ((a*rs+b)*rs+c)*rs+d   # a VECTOR of k's

    #ptd <- k*tmp + cs + TX - .5 
    #ptd <- apply( rbind(tmp,ks), 2, function( inp ) { return( inp[3]*inp[1:2] + cs + TX - .5 )} ) + as.integer(one.based.output)
    ptd <- t(t(tmp)*ks) + cs + TX - .5 + as.integer(one.based.output)
    return(ptd)
}
