# radialUndistortedToDistortedOpenCV.q v1.0
#' Convert undistorted to radially distorted OpenCV 5-param.
##############################################################################
#' Given undistorted image coords and distortion parameters,
#'  returns the corresponding image coords in the distorted image.
#' Parrots openCV model [eqn (1) in details].
#'
#' @inheritParams radialDistortedToUndistortedFulla
#' @param k1 radial distortion parameter, barrel distort/r^2 coeffient (see details)
#' @param k2 radial distortion parameter, r^4 coefficient (see details)
#' @param k3 radial distortion parameter, r^6 coefficient (see details); 
#'            often 0 unless calculated from many images 
#'            (otherwise can introduce more noise than it corrects)
#' @param p1 tangential distortion parameter (see details); default 0
#' @param p2 tangential distortion parameter (see details); default 0
#' @param fx focal length in terms of pixels; f (mm) * scale_x (px/mm)
#' @param fy focal length in terms of pixels; f (mm) * scale_y (px/mm)
#' @param cx x-centre of image; usually (nx-1)/2
#' @param cy y-centre of image; usually (ny-1)/2
#' @param cxscale x-centre of image with scale (see details); if larger than cx the original image will be shrunk and more distortion will show. default cx.
#' @param cyscale y-centre of image with scale (see details); default cy.
#' @param fxscale normalisation factor for x coords in output image (see details); default fx
#' @param fyscale normalisation factor for y coords in output image (see details); default fy
#' @return pts_d Distorted pixel coords (not necessarily integers) corresponding to the input ones.
#'               It is up to the user to interpolate them.
#
# Details:
#' @details
#'   This function implements the equation (same as OpenCV/Camera Calibration Toolbox):
#'
#'   x_[distorted] := x(1 + k1 r^2 + k2 r^4 + k3 r^6) + 2p1xy + p2(r^2+2x^2)  ------ (1a)
#'   y_[distorted] := y(1 + k1 r^2 + k2 r^4 + k3 r^6) + 2p2xy + p1(r^2+2y^2)  ------ (1b)
#'
#'   where r is the location of a point in the UNdistorted/corrected image,
#'    being the normalised radius from the image centre.
#'   The normalisation factors are fx and fy, usually being half the image's dimension such that
#'    when x/fx or y/fx = 1, one is at the image edge.
#'
#'   So, given points (u,v) in the UNDISTORTED image, the corresponding distorted points (u',v') 
#'    are given by:
#' 
#'      x' <- (u - cxscale)/fxscale                      # normalise coords
#'      y' <- (v - cyscale)/fyscale
#'
#'      r2        <- x'^2 + y'^2
#'  
#'      k         <- (1 + k1(r2) + k2(r2)^2 + k3(r2)^3)  # scale factor
#'      dx        <- 2p1*x*y + p2(r2 + 2x^2)             # tangential in x
#'      dy        <- 2p2*x*y + p1(r2 + 2y^2)             # tangential in y
#'
#'      u'        <- (k*x' + dx)fx + cx        # scale by k and shift back.
#'      v'        <- (k*y' + dx)fx + cx        # scale by k and shift back.
#'   
#'   The return value of the function is (u',v'), the distorted pix coords.
#'
#'   Then the intensity at location (x,y) in the undistorted image should be the intensity
#'    at location (u',v') in the distorted image.
#'   Interpolate it how you will.
#'
#'   Note that fxscale and cxscale have the effect of changing the field of view
#'    of the output image: for example when c.scale == c. and f.scale == f. then
#'    the output image contains no unmapped pixels and is the same size as the input.
#'   One can have c.scale and f.scale larger than c. and f. such that the output image
#'    shows the distorted edges of the input image and has unmapped pixels.
#' 
#'   To calculate these new values one uses the openCV function cv::getOptimalNewCameraMatrix().
#'
# See Also:
#' @references
#' Implementation in C++ (OpenCV): function InitUndistortRectifyMap
#'     http://opencv.willowgarage.com/documentation/python/calib3d_camera_calibration_and_3d_reconstruction.html#initundistortrectifymap
#' The camera calibration toolbox for a good description of the model:
#'     http://www.vision.caltech.edu/bouguetj/calib_doc/htmls/parameters.html
#' Also, the corresponding OpenCV page:
#'     http://opencv.willowgarage.com/documentation/python/camera_calibration_and_3d_reconstruction.html
#' Z. Zhang. "A flexible new technique for camera calibration".
#'     IEEE Transactions on Pattern Analysis and Machine Intelligence, 22(11):1330-1334, 2000.
#
# Related Functions:
#   unRadialDistortOpenCV              -- removes radial distortion in an entire image.
#
#' @family radialDistortion
#' @export
# AC 7 Nov 2011
##############################################################################
#' @section History:
#' v1.0 (7-11-2011)
##############################################################################
# cat(' pts_d <- radialUndistortedToDistortedOpenCV <- function( i_pts, k1,k2,k3,p1,p2,fx,fy,cx,cy, one.based.input=FALSE, one.based.output=FALSE )\n')
radialUndistortedToDistortedOpenCV <- function( i_pts,k1,k2,k3,p1,p2,cx,cy,fx,fy,cxscale=cx,cyscale=cy,fxscale=fx,fyscale=fy, one.based.input=FALSE, one.based.output=FALSE )
{
    if ( k1 == 0 && k2 == 0 && k3 == 0 && p1 == 0 && p2 == 0 )
        return( oneBasedSwitch(i_pts, one.based.input, one.based.output) )
    pts <- i_pts - as.integer(one.based.input) # convert to zero-based.
    D <- dim(i_pts)
    if ( D[1] != 2 && D[2] == 2 ) pts <- t(i_pts) # flip it if it's the wrong way round.
    
    #tmp <- (pts - c(cx,cy))/c(fx,fy)
    tmp <- (pts - c(cxscale,cyscale))/c(fxscale,fyscale)
    r2s <- colSums(tmp^2) # a VECTOR of r^2s
    ks  <- ((k3*r2s+k2)*r2s+k1)*r2s+1   # a VECTOR of k's
    if ( p1 == 0 && p2 == 0 ) {
        dxs <- 0
    } else {
        t2 <- tmp[1,]*tmp[2,]
        dxs <- 2*c(p1,p2)*rbind(t2,t2) + c(p2,p1)*(r2s+2*tmp^2)
    }

    #ptd <- (  tmp[1:2,] * rbind(ks,ks) + dxs )*c(fxscale,fyscale) + c(cxscale,cyscale) + as.integer(one.based.output)
    ptd <- (  tmp[1:2,] * rbind(ks,ks) + dxs )*c(fx,fy) + c(cx,cy) + as.integer(one.based.output)
    return(ptd)
}
