# unRadialDistortFulla.q v1.1
#' removes radial distortion from an image using Fulla/Hugin model.
##############################################################################
#' Given an image and parameters a,b,c,d,e,f :
#'  un-distorts an image that has been barrel distorted,
#'  using the same model/algorithm as FULLA (command line prog)
#
# Usage:
#       outImg <- unRadialDistortFulla( img, a=0, b=0, c=0, d=1, e=0, f=0, defaultValue=0 )
#   where:
#' @inheritParams radialDistortedToUndistortedFulla
#' @param imgPath      image file name to be corrected
#' @param defaultValue the value of pixels that have no mapping in the original image (default black)
#
# TODO : simulate the scale factor that avoids black borders (-s option in fulla)
#
# Details:
#' @details
#'   Hugin/Panotools uses the model
#'
#'       [xy]_{distorted} = [xy](d + cr + br^2 + ar^3),                 ---------- (1)
#'
#'   where r is the location of a point in the UNdistorted/corrected image,
#'    being the normalised radius from the image centre.
#'   Also, d is often chosen to be 1-a-b-c, as this preserves image size: 
#'    when r = 1, then the distorted r is also 1.
#'
#'   The normalised radius is such that the largest circle fitting entirely in the
#'    image has r=1.
#'  
#'   That is, given a (0-based) pixel index (x,y) in the distorted/uncorrected image,
#'    one calculates the radius r by:
#'
#'      c_x       <- width/2                 # location of image centre before shifting
#'      c_y       <- height/2 
#'
#'      x_shifted <- x - c_x - e + 0.5       # add 0.5 because of centre/edge of pixel thing.
#'      y_shifted <- y - c_y - f + 0.5
#'
#'      R         <- MIN( width/2, height/2 )
#'      r         <- sqrt( x_shifted^2 + y_shifted^2 ) / R
#'  
#'      k         <- (d + cr + br^2 + ar^3)  # scale factor
#'
#'      xd        <- k*x_shifted + c_x + e - 0.5      # scale by k and shift back.
#'      yd        <- k*y_shifted + c_y + f - 0.5      
#' 
#'   Then the intensity at location (x,y) in the undistorted image should be the intensity
#'    at location (xd,yd) in the distorted image.
#'   Interpolate it how you will.
#
# See Also:
#' @references
#'   For the model, fulla source is incredibly helpful, and also the sites
#'       http://wiki.panotools.org/Lens_Correction_in_PanoTools
#'       http://wiki.panotools.org/Lens_correction_model
#'
#'   Someone has implemented these in Matlab (slightly different method???)
#'       http://staff.science.uva.nl/~iesteban/
#'
#'   For a way to determine a,b,c,d you need to have some sort of calibration image with known
#'    straight lines in it, then select points on that line, and basically optimise
#'    equation (1) with respect to a,b,c,d so that the undistorted line is straight.
#'
#'   Hugin is a useful tool for doing this, you essentially pick points on straight lines and optimise
#'    with respect to a,b,c only (some call for y,p,r first, then a,b,c ?)
#'   I only optimised w.r.t. b so far (that is the parameter that corresponds directly to barrel distortion).
#'
#'   NOTE: put d = 1 - a - b - c? this means that when r=1, r_dist = 1 -- preserves image size.
#
#' @family radialDistortion
#' @export
# Related Functions:
#   radialDistortedToUndistortedFulla, radialUndistortedToDistortedFulla.
#
# AC 5 July 2011
##############################################################################
#' @section History:
#' v1.1 (7-11-2011_
#'     - added documentation
#' v1.0 (5-7-2011)
##############################################################################
# cat('  outImg <- unRadialDistortFulla( img, a=0,b=-0.02,c=0 )\n')
unRadialDistortFulla <- function( imgPath, a=0, b=0, c=0, d=1-a-b-c, e=0, f=0, defaultValue=0 )
{
    if ( is.character(imgPath) ) 
        img <- imload(imgPath)
    else if ( is.image(imgPath) )
        img <- imgPath
    else
        stop("Unrecognised image")

    if ( a==0&&b==0&&c==0&&d==1 )
        return(img)

    nx <- imgetnx(img,1)
    ny <- imgetny(img,1)
    nc <- imgetnumcomp(img)

    # NOTE: need (X,Y) hence imtranspose.
    if ( is.array(img) )
        imgAry <- imtranspose(img)
    else
        imgAry <- im2array(img) 
    imgAry <- imexpand(imgAry)


    # ARGH: TRANSPOSED IMAGE
    # Initialise output image, fixed at same size of input image.
    outImgAry <- array( defaultValue, dim=c(nx,ny,nc) )

    # generate list of coordinates in the output image.
    XYs <- matrix( c( rep( 0:(nx-1), ny), rep( 0:(ny-1),each=nx) ), ncol=2 )

    # transform them (could take a while!)
    XYs_dist <- t(radialUndistortedToDistortedFulla( XYs, a,b,c,d, nx,ny, e,f, one.based.output=T ))

    # assign output intensities, don't forget R is 1-based
    # I know there's a faster way but can't for the life of me think of it.
    for ( c in 1:nc ) 
        outImgAry[ cbind(XYs+1,c) ] <- imgAry[ cbind( round(XYs_dist), c ) ]

    return(array2im(imsqueeze(outImgAry),
                    outPixType=imgetpixtype(img,1),transpose=TRUE))
}
