# unRadialDistortOpenCV.q
#' removes radial distortion from an image using OpenCV model.
##############################################################################
#' This is basically a port of the InitUndistortRectifyMap function in OpenCV.
#'
#' @inheritParams radialUndistortedToDistortedOpenCV
#' @param imgPath      image file name to be corrected
#' @param defaultValue the value of pixels that have no mapping in the original image (default black)
#' @return         the input image, undistorted, same size as output image.
#'
#' @details The OpenCV/Camera Calibration Toolbox model for lens distortion is described in
#'  a number of papers, e.g. Heikkil & Silven.
#'
#' It is as follows:
#' s [u,v,1]^T = A [x,y,z]^T        ------- (1)
#' s : some sort of scale factor b/t pixels and world units
#' [u,v,1]^T  position of a point in pixel coordinates
#' A : intrinsic camera parameters, 3x3 matrix
#' [x y z]^T position of the point in world space after translation, rotation.
#'
#' That is,
#' [x y z]^T = R [X Y Z]^T + t
#' 
#' R : 3x3 rotation matrix of camera
#' t : translation of camera.
#' [X Y Z]^T : position of the point in world space.
#'
#' A has the following form:
#'     / fx 0  cx \
#' A = | 0  fy cy |
#'     \ 0  0  1  /
#' fx, fy : focal lengths in x/y dimension in pixel equivalent
#' cx, cy : centre of image.
#' A depends on the image size and must be scaled up and down with it,
#'  and also on the focal length, but otherwise nothing else: it is
#'  camera intrinsic.
#'
#' Make the following rearrangement of (1), assuming z != 0:
#' s [u v 1]^T = zA [x' y' 1]^T,
#' where:
#' x' = x/z
#' y' = y/z
#'
#' Then, one obtains:
#' u = fx x' + cx                 ---------- (2a)
#' v = fy y' + cy                 ---------- (2b) 
#' [ z == s ??]
#'
#' Since the world is not perfect there is lens distortion in the image.
#' There is both a radial and tangential component.
#' That is:
#' x'   = x/z   [as before]
#' y'   = y/z   [as before]
#' x'' := x'(1 + k_1 r^2 + k_2 r^4 + k_3 r^6) + 2p_1x'y' + p_2(r^2+2x'^2)  ------ (3a)
#' y'' := y'(1 + k_1 r^2 + k_2 r^4 + k_3 r^6) + 2p_2x'y' + p_1(r^2+2y'^2)  ------ (3b)
#' u    = fx x'' + cx             ---------- (4a)
#' v    = fy y'' + cy             ---------- (4b)
#'
#' Eqns (3) have two components, the radial (1+k_1r^2+...) and tangential (p_1,p_2).
#' If one sets k_3 == 0, one obtains the model of Zhang. This is often done, especially
#'  if there are not enough calibration images to be sure of k_3 (it may introduce more
#'  confusion into the model than it corrects)
#' Also, p_1 & p_2 are often set to 0 as such effects are usually negligible.
#'
#' NOTE:
#' Hugin uses the model:
#' x'' = x'(d + c r + b r^2 + a r^3)
#' y'' = y'(d + c r + b r^2 + a r^3)
#' Which I can't find any mathematical basis for.
#
#' @references
#' Implementation in C++ (OpenCV): function InitUndistortRectifyMap
#'     http://opencv.willowgarage.com/documentation/python/calib3d_camera_calibration_and_3d_reconstruction.html#initundistortrectifymap
#' The camera calibration toolbox for a good description of the model:
#'     http://www.vision.caltech.edu/bouguetj/calib_doc/htmls/parameters.html
#' Also, the corresponding OpenCV page:
#'     http://opencv.willowgarage.com/documentation/python/camera_calibration_and_3d_reconstruction.html
#' Z. Zhang. "A flexible new technique for camera calibration".
#'     IEEE Transactions on Pattern Analysis and Machine Intelligence, 22(11):1330-1334, 2000.
#' @family radialDistortion
#' @export
#
# AC 7/11/11
##############################################################################
unRadialDistortOpenCV <- function( imgPath, k1=0,k2=0,k3=0,p1=0,p2=0,cx=-1,cy=-1,fx=-1,fy=-1,cxscale=cx,cyscale=cy,fxscale=fx,fyscale=fy,defaultValue=0 )
{
    if ( is.character(imgPath) ) 
        img <- imload(imgPath)
    else if ( is.image(imgPath) )
        img <- imgPath
    else
        stop("Unrecognised image")

    # THEN DON'T EVEN SHIFT !!!!
    if ( k1==0&&k2==0&&k3==0&&p1==0&&p2==0&&cx==(nx-1)/2&&cy==(ny-1)/2 ) 
        return(img)


    nx <- imgetnx(img,1)
    ny <- imgetny(img,1)
    nc <- imgetnumcomp(img)

    if ( is.array(img) )
        imgAry <- imtranspose(img)
    else
        imgAry <- im2array(img)
    imgAry <- imexpand(imgAry)



    # 0-based 
    if ( cx <= 0 )
       cx <- (nx-1)/2
    if ( cy <= 0 )
       cy <- (ny-1)/2
    if ( fx <= 0||fy <= 0 ) {
        # really unsure if this is a sensible guess
        # "focal length in pixels": f(mm) * scale_factor (px/mm)
        fx <- min(cx,cy)
        fy <- fx*ny/nx
    }
    if ( cxscale <= 0 )
        cxscale <- cx
    if ( cyscale <= 0 )
        cyscale <- cy
    if ( fxscale <= 0 )
        fxscale <- fx
    if ( fyscale <= 0 )
        fyscale <- fy

    # Initialise output image, fixed at same size of input image.
    outImgAry <- array(defaultValue,dim=c(nx,ny,nc))

    # generate list of coordinates in the output image.
    XYs <- matrix( c( rep( 0:(nx-1), ny), rep( 0:(ny-1),each=nx) ), ncol=2 )

    # transform them (could take a while!)
    XYs_undist <- t(radialUndistortedToDistortedOpenCV( XYs, k1,k2,k3,
                                                       p1,p2,
                                                       cx,cy,fx,fy,
                                                       cxscale,cyscale,
                                                       fxscale,fyscale,
                                                       one.based.input=F,
                                                       one.based.output=T))

    # assign output intensities, don't forget R is 1-based
    # I know there's a faster way but can't for the life of me think of it.
    XYs_undist <- pmax(round(XYs_undist),1)
    XYs_undist[,1] <- pmin(XYs_undist[,1],nx)
    XYs_undist[,2] <- pmin(XYs_undist[,2],ny)

    for ( c in 1:nc ) 
        outImgAry[ cbind(XYs+1,c) ] <- imgAry[ cbind( XYs_undist, c ) ]

    if ( nc == 1 ) outImgAry <- outImgAry[,,1]

    return(array2im(imsqueeze(outImgAry),
                    outPixType=imgetpixtype(img,1),transpose=TRUE))

}
