\name{radialDistortedToUndistortedFulla}
\alias{radialDistortedToUndistortedFulla}
\title{Convert radially distorted to undistorted coords w/ Hugin/Fulla.
Given distorted image coords (0-based) and distortion parameters,
 returns the corresponding image coords in the UNdistorted image.
Parrots fulla, i.e. solves (1b) for r.}
\usage{
  radialDistortedToUndistortedFulla(i_pts, a, b, c, d, nx,
    ny, e = 0, f = 0, one.based.input = F,
    one.based.output = F)
}
\arguments{
  \item{a}{parameter for distortion polynomial, r^3 term
  (see details)}

  \item{b}{parameter for distortion polynomial, barrel
  distort/r^2 term (see details)}

  \item{c}{parameter for distortion polynomial, r term (see
  details)}

  \item{d}{parameter for distortion polynomial, scaling
  term (see details)}

  \item{nx}{dimension of input image (x) (only handles same
  size input/output image at the moment)}

  \item{ny}{dimension of input image (y) (only handles same
  size input/output image at the moment)}

  \item{e}{the x shift of optical centre in the image
  Default image centre, leave if you don't know.}

  \item{f}{the y shift of optical centre in the image
  Default image centre, leave if you don't know.}

  \item{i_pts}{image coordinates for the distorted image
  (0-based).  These should be an array/matrix of 2x[however
  many points], with x on the top and y on the bottom (or
  left/right).}

  \item{one.based.input}{boolean, whether input coords are
  1-based or 0-based. Default 0-based.}

  \item{one.based.output}{boolean, whether output coords
  are to be 1-based or 0-based. Default 0-based.}
}
\value{
  pts_u 0-based pixel coords in the undistorted image
  corresponding to the distorted ones provided. Matrix,
  2x[npoints].
}
\description{
  The outputs are not necessarily integers.
}
\details{
  This function solves following equation for r (same as
  hugin/panotools):

  [xy]_{distorted} = [xy](d + cr + br^2 + ar^3), ----------
  (1a) equivalently r_{distorted} = r(d + cr + br^2 +
  ar^3), ---------- (1b)

  where r is the location of a point in the
  UNdistorted/corrected image, being the normalised radius
  from the image centre.  Also, d is often chosen to be
  1-a-b-c, as this preserves image size: when r = 1, then
  the distorted r is also 1.

  That is, given r_{distorted}( xd, yd ), it finds r such
  that (1b) holds.

  Once this r found, the scaling factor (d+cr+br^2+ar^3) in
  (1a) is calculated by rearranging (1b): it is equal to
  r_{distorted}/r.

  Then (1a) is used to calculated [xy] by

  [xy] = [xy]_{distorted} * r/r_{distorted} ---------- (2)

  r is a normalised radius, such that the largest circle
  fitting entirely in the image has r=1.

  That is, given a (0-based) pixel index (xd,yd) in the
  distorted/uncorrected image, one calculates the radius rd
  by:

  c_x <- width/2 # location of image centre before shifting
  c_y <- height/2

  xd_shifted <- xd - c_x - e + 0.5 # add 0.5 because of
  centre/edge of pixel thing.  yd_shifted <- yd - c_y - f +
  0.5

  R <- MIN( width/2, height/2 ) rd <- sqrt( xd_shifted^2 +
  yd_shifted^2 ) / R r <- { SOLVE (1b) for r }

  k <- r/rd # [ this is just 1/(d + cr + br^2 + ar^3) ]

  x <- k*xd_shifted + c_x + e - 0.5 # scale by k and shift
  back.  y <- k*yd_shifted + c_y + f - 0.5

  The return value of the function is (xd,yd), the
  distorted pix coords.

  Then the intensity at location (x,y) in the undistorted
  image should be the intensity at location (xd,yd) in the
  distorted image.  Interpolate it how you will.
}
\section{History}{
  v1.2 (7-11-2011) - added doxygen - added one based
  input/output options. v1.1 (6-7-2011) - vectorising was
  broken, fixed it. v1.0 (5-7-2011)
}
\references{
  For the model, fulla source is incredibly helpful, and
  also the sites
  http://wiki.panotools.org/Lens_Correction_in_PanoTools
  http://wiki.panotools.org/Lens_correction_model

  Someone has implemented these in Matlab (slightly
  different method???)
  http://staff.science.uva.nl/~iesteban/

  For a way to determine a,b,c,d you need to have some sort
  of calibration image with known straight lines in it,
  then select points on that line, and basically optimise
  equation (1) with respect to a,b,c,d so that the
  undistorted line is straight.

  Hugin is a useful tool for doing this, you essentially
  pick points on straight lines and optimise with respect
  to a,b,c only (some call for y,p,r first, then a,b,c ?)

  NOTE: put d = 1 - a - b - c? this means that when r=1,
  r_dist = 1 -- preserves image size.
}
\seealso{
  Other radialDistortion: \code{\link{fullaParameters}},
  \code{\link{openCVParameters}},
  \code{\link{radialDistortedToUndistortedOpenCV}},
  \code{\link{radialUndistortedToDistortedFulla}},
  \code{\link{radialUndistortedToDistortedOpenCV}},
  \code{\link{rLensDistortion}},
  \code{\link{unRadialDistortFulla}},
  \code{\link{unRadialDistortOpenCV}}
}

