\name{unRadialDistortOpenCV}
\alias{unRadialDistortOpenCV}
\title{removes radial distortion from an image using OpenCV model.
This is basically a port of the InitUndistortRectifyMap function in OpenCV.}
\usage{
  unRadialDistortOpenCV(imgPath, k1 = 0, k2 = 0, k3 = 0,
    p1 = 0, p2 = 0, cx = -1, cy = -1, fx = -1, fy = -1,
    cxscale = cx, cyscale = cy, fxscale = fx, fyscale = fy,
    defaultValue = 0)
}
\arguments{
  \item{imgPath}{image file name to be corrected}

  \item{defaultValue}{the value of pixels that have no
  mapping in the original image (default black)}

  \item{k1}{radial distortion parameter, barrel distort/r^2
  coeffient (see details)}

  \item{k2}{radial distortion parameter, r^4 coefficient
  (see details)}

  \item{k3}{radial distortion parameter, r^6 coefficient
  (see details); often 0 unless calculated from many images
  (otherwise can introduce more noise than it corrects)}

  \item{p1}{tangential distortion parameter (see details);
  default 0}

  \item{p2}{tangential distortion parameter (see details);
  default 0}

  \item{cx}{x-centre of image; usually (nx-1)/2}

  \item{cy}{y-centre of image; usually (ny-1)/2}

  \item{fx}{focal length in terms of pixels; f (mm) *
  scale_x (px/mm)}

  \item{fy}{focal length in terms of pixels; f (mm) *
  scale_y (px/mm)}

  \item{cxscale}{x-centre of image with scale (see
  details); if larger than cx the original image will be
  shrunk and more distortion will show. default cx.}

  \item{cyscale}{y-centre of image with scale (see
  details); default cy.}

  \item{fxscale}{normalisation factor for x coords in
  output image (see details); default fx}

  \item{fyscale}{normalisation factor for y coords in
  output image (see details); default fy}
}
\value{
  the input image, undistorted, same size as output image.
}
\description{
  removes radial distortion from an image using OpenCV
  model. This is basically a port of the
  InitUndistortRectifyMap function in OpenCV.
}
\details{
  The OpenCV/Camera Calibration Toolbox model for lens
  distortion is described in a number of papers, e.g.
  Heikkil & Silven.

  It is as follows: s [u,v,1]^T = A [x,y,z]^T ------- (1) s
  : some sort of scale factor b/t pixels and world units
  [u,v,1]^T position of a point in pixel coordinates A :
  intrinsic camera parameters, 3x3 matrix [x y z]^T
  position of the point in world space after translation,
  rotation.

  That is, [x y z]^T = R [X Y Z]^T + t

  R : 3x3 rotation matrix of camera t : translation of
  camera. [X Y Z]^T : position of the point in world space.

  A has the following form: / fx 0 cx \ A = | 0 fy cy | \ 0
  0 1 / fx, fy : focal lengths in x/y dimension in pixel
  equivalent cx, cy : centre of image. A depends on the
  image size and must be scaled up and down with it, and
  also on the focal length, but otherwise nothing else: it
  is camera intrinsic.

  Make the following rearrangement of (1), assuming z != 0:
  s [u v 1]^T = zA [x' y' 1]^T, where: x' = x/z y' = y/z

  Then, one obtains: u = fx x' + cx ---------- (2a) v = fy
  y' + cy ---------- (2b) [ z == s ??]

  Since the world is not perfect there is lens distortion
  in the image. There is both a radial and tangential
  component. That is: x' = x/z [as before] y' = y/z [as
  before] x'' := x'(1 + k_1 r^2 + k_2 r^4 + k_3 r^6) +
  2p_1x'y' + p_2(r^2+2x'^2) ------ (3a) y'' := y'(1 + k_1
  r^2 + k_2 r^4 + k_3 r^6) + 2p_2x'y' + p_1(r^2+2y'^2)
  ------ (3b) u = fx x'' + cx ---------- (4a) v = fy y'' +
  cy ---------- (4b)

  Eqns (3) have two components, the radial (1+k_1r^2+...)
  and tangential (p_1,p_2). If one sets k_3 == 0, one
  obtains the model of Zhang. This is often done,
  especially if there are not enough calibration images to
  be sure of k_3 (it may introduce more confusion into the
  model than it corrects) Also, p_1 & p_2 are often set to
  0 as such effects are usually negligible.

  NOTE: Hugin uses the model: x'' = x'(d + c r + b r^2 + a
  r^3) y'' = y'(d + c r + b r^2 + a r^3) Which I can't find
  any mathematical basis for.
}
\references{
  Implementation in C++ (OpenCV): function
  InitUndistortRectifyMap
  http://opencv.willowgarage.com/documentation/python/calib3d_camera_calibration_and_3d_reconstruction.html#initundistortrectifymap
  The camera calibration toolbox for a good description of
  the model:
  http://www.vision.caltech.edu/bouguetj/calib_doc/htmls/parameters.html
  Also, the corresponding OpenCV page:
  http://opencv.willowgarage.com/documentation/python/camera_calibration_and_3d_reconstruction.html
  Z. Zhang. "A flexible new technique for camera
  calibration".  IEEE Transactions on Pattern Analysis and
  Machine Intelligence, 22(11):1330-1334, 2000.
}
\seealso{
  Other radialDistortion: \code{\link{fullaParameters}},
  \code{\link{openCVParameters}},
  \code{\link{radialDistortedToUndistortedFulla}},
  \code{\link{radialDistortedToUndistortedOpenCV}},
  \code{\link{radialUndistortedToDistortedFulla}},
  \code{\link{radialUndistortedToDistortedOpenCV}},
  \code{\link{rLensDistortion}},
  \code{\link{unRadialDistortFulla}}
}

